/*
 *  np_print.c
 *  NtileCodePredictor
 *
 *  Created by Naoto Morikawa on 2/23/07.
 *  Copyright 2007 (C) 2007 Naoto Morikawa. All rights reserved.
 *
 *  2007-03-05 version 0.1 release
 *  2007-04-10 support the design-protein option, "-d", in print_np_protein()
 *  2007-04-10 create print_code_fragment() and print_code_frag_sequence() 
 *              to support the "-d" option
 *  2007-04-10 make an adjustment of score values in print_prediction()
 *  2007-04-15 version 0.2 release
 *
 */

#include <stdio.h>
#include "np_main.h"

extern int int_to_aachr(int val);	// np_search_table.c
extern struct np_settings my_settings;	// NM 2007-04-10


int int_to_codechr(int val)
{
	switch (val) {
		case (ST_0):
			return '0';
		case (ST_1):
			return '1';
		case (ST_2):
			return '2';
		case (ST_3):
			return '3';
		case (ST_4):
			return '4';
		case (ST_5):
			return '5';
		case (ST_6):
			return '6';
		case (ST_7):
			return '7';
		case (ST_8):
			return '8';
		case (ST_9):
			return '9';
		case (ST_A):
			return 'A';
		case (ST_B):
			return 'B';
		case (ST_C):
			return 'C';
		case (ST_D):
			return 'D';
		case (ST_E):
			return 'E';
		case (ST_F):
			return 'F';
		case (ST_G):
			return 'G';
		case (ST_H):
			return 'H';
		case (ST_I):
			return 'I';
		case (ST_J):
			return 'J';
		case (ST_K):
			return 'K';
		case (ST_L):
			return 'L';
		case (ST_M):
			return 'M';
		case (ST_N):
			return 'N';
		case (ST_O):
			return 'O';
		case (ST_P):
			return 'P';
		case (ST_Q):
			return 'Q';
		case (ST_R):
			return 'R';
		case (ST_S):
			return 'S';
		case (ST_T):
			return 'T';
		case (ST_U):
			return 'U';
		case (ST_V):
			return 'V';
		case (ST_UNK):
			return '?';
		case (ST_UNDEF):
			return '-';
		default:
			// fprintf(STDERR, "int_to_codechr() : ERROR invalid ntile code value = %d\n", val);
			return 'Z';
	}
}

void print_code_frag_sequence(struct np_fragment * pt_fragment, struct np_protein * pt_protein)
{
	int aa_ind;
	
	fprintf(STDOUT, "Code fragment : 5-tile code sequence = \"");
	for (aa_ind = pt_fragment->aa_ct;
	     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
	     aa_ind++) {
		fprintf(STDOUT, "%c", int_to_codechr(pt_protein->aa[aa_ind].actual));
	}
	fprintf(STDOUT, "\"\n");
}

void print_code_fragment(struct np_fragment * pt_fragment, struct np_protein * pt_protein)
{
	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "Code fragment : from the %d-th amino acid in chain %d  (length = %2d)\n",
		pt_fragment->aa_ct, pt_fragment->chn_ct, pt_fragment->length);
	print_code_frag_sequence(pt_fragment, pt_protein);
}

void print_frag_sequence(struct np_fragment * pt_fragment, struct np_protein * pt_protein)
{
	int aa_ind;
	
	fprintf(STDOUT, "Fragment : aa sequence = \"");
	for (aa_ind = pt_fragment->aa_ct;
	     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
	     aa_ind++) {
		fprintf(STDOUT, "%c", int_to_aachr(pt_protein->aa[aa_ind].name));
	}
	fprintf(STDOUT, "\"\n");
}

void print_fragment(struct np_fragment * pt_fragment, struct np_protein * pt_protein)
{
	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "Fragment : from the %d-th amino acid in chain %d  (length = %2d)\n",
		pt_fragment->aa_ct, pt_fragment->chn_ct, pt_fragment->length);
	print_frag_sequence(pt_fragment, pt_protein);
}

void print_score_table(int aa_ind, struct np_protein * pt_protein)
{
	int tbl_ind;
	int chn_ind;
	int aa_pos_offset;
	
	chn_ind = pt_protein->aa[aa_ind].chain_id;
	aa_pos_offset = pt_protein->chain[chn_ind].aa_head - 1;
	
	fprintf(STDOUT, "Score table:   chn%2d  aa[%5d] ", chn_ind,
		aa_ind - aa_pos_offset);
	for (tbl_ind = 0; tbl_ind < SCORE_TABLE_SIZE; tbl_ind++) {
		if (pt_protein->aa[aa_ind].score_tbl[tbl_ind] > 0.0)
			fprintf(STDOUT, "   %c  ", int_to_codechr(tbl_ind));
	}
	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "Score table:   chn%2d  aa[%5d] ", chn_ind,
		aa_ind - aa_pos_offset);
	for (tbl_ind = 0; tbl_ind < SCORE_TABLE_SIZE; tbl_ind++) {
		if (pt_protein->aa[aa_ind].score_tbl[tbl_ind] > 0.0)
			fprintf(STDOUT, "%6.2f",
				pt_protein->aa[aa_ind].score_tbl[tbl_ind]);
	}
	fprintf(STDOUT, "\n");
}

void print_prediction(int aa_ind, struct np_protein * pt_protein)
{
	int tbl_ind;
	int chn_ind;
	int aa_pos_offset;
	int table_size;		// NM 2007-04-10
	float adjustment;	// NM 2007-04-10
	
	if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
		table_size = ST_UNK;
	} else {
		table_size = L_UNK;
	}
	if (pt_protein->aa[aa_ind].num_valid_scores > 99.99) {	// NM 2007-04-10
		adjustment = pt_protein->aa[aa_ind].num_valid_scores;
	} else {
		adjustment = 1.0;
	}
	chn_ind = pt_protein->aa[aa_ind].chain_id;
	aa_pos_offset = pt_protein->chain[chn_ind].aa_head - 1;
	if (0) {
		fprintf(STDOUT, "Prediction :   chn%2d  aa[%5d] ", chn_ind,
			aa_ind - aa_pos_offset);
		for (tbl_ind = 0; tbl_ind < SCORE_TABLE_SIZE; tbl_ind++) {
			if (PRED_SCORE(aa_ind, tbl_ind) > 0.0)
				fprintf(STDOUT, "%3d   ", tbl_ind);
		}
		fprintf(STDOUT, "\n");
	}
	fprintf(STDOUT, "Prediction :   chn%2d  aa[%5d]  ", chn_ind,	// NM 2007-04-10
		aa_ind - aa_pos_offset);
	for (tbl_ind = 0; tbl_ind < table_size; tbl_ind++) {
		if (PRED_SCORE(aa_ind, tbl_ind) > 0.0) {
			if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
				fprintf(STDOUT, "  %c  >",
					int_to_codechr(PRED_CODE(aa_ind, tbl_ind)));
			} else {
				fprintf(STDOUT, "  %c  >",
					int_to_aachr(PRED_CODE(aa_ind, tbl_ind)));
			}
		}
	}
	fprintf(STDOUT, "     (");	// NM 2007-04-10
	tbl_ind = table_size;		// NM 2007-04-10
	for (/* tbl_ind = ST_UNK */; tbl_ind < SCORE_TABLE_SIZE; tbl_ind++) {	// NM 2007-04-10
		if (PRED_SCORE(aa_ind, tbl_ind) > 0.0) {
			if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
				fprintf(STDOUT, "   %c  ",		// NM 2007-04-10
					int_to_codechr(PRED_CODE(aa_ind, tbl_ind)));
			} else {
				fprintf(STDOUT, "   %c  ",		// NM 2007-04-10
					int_to_aachr(PRED_CODE(aa_ind, tbl_ind)));
			}
		}
	}
	fprintf(STDOUT, " )\n");
	fprintf(STDOUT, "Prediction :   chn%2d  aa[%5d] ", chn_ind,
		aa_ind - aa_pos_offset);
	for (tbl_ind = 0; tbl_ind < table_size; tbl_ind++) {
		if (PRED_SCORE(aa_ind, tbl_ind) > 0.0)
			fprintf(STDOUT, "%6.2f", PRED_SCORE(aa_ind, tbl_ind) / adjustment);
	}
	fprintf(STDOUT, "      (");
	tbl_ind = table_size;	// NM 2007-04-10
	for (/* tbl_ind = ST_UNK */; tbl_ind < SCORE_TABLE_SIZE; tbl_ind++) {	// NM 2007-04-10
		if (PRED_SCORE(aa_ind, tbl_ind) > 0.0)
			fprintf(STDOUT, "%6.2f", PRED_SCORE(aa_ind, tbl_ind) / adjustment);
	}
	fprintf(STDOUT, " )\n");
}

void print_np_protein(struct np_protein * pt_protein, BOOLEAN print_score, BOOLEAN print_predict)
{
	int chn_ct, aa_ct;
	int aa_pos_offset;
	
	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "Np_protein : <Protein  chains=\"%5d\" amino acids=\"%10d\" coverage=\"%6.4f\">\n",
		pt_protein->chain_num, pt_protein->aa_num, pt_protein->coverage);
	
	for (chn_ct = pt_protein->chain_head;
	     chn_ct <= pt_protein->chain_tail; chn_ct++) {
		fprintf(STDOUT, "Np_protein :\n");
		fprintf(STDOUT, "Np_protein : <Chain %d  amino acids=\"%10d\"  coverage=\"%6.4f\">\n",
			chn_ct, pt_protein->chain[chn_ct].aa_num, pt_protein->chain[chn_ct].coverage);
		aa_pos_offset = pt_protein->chain[chn_ct].aa_head - 1;
		for (aa_ct = pt_protein->chain[chn_ct].aa_head;
		     aa_ct <= pt_protein->chain[chn_ct].aa_tail;
		     aa_ct++) {
			if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
				fprintf(STDOUT, "Np_protein : <Amico acid %d  name=\"%c\" chain=\"%d\">\n",
					aa_ct - aa_pos_offset,
					int_to_aachr(pt_protein->aa[aa_ct].name),
					pt_protein->aa[aa_ct].chain_id);
			} else {
				fprintf(STDOUT, "Np_protein : <Amico acid %d  code=\"%c\" chain=\"%d\">\n",
					aa_ct - aa_pos_offset,
					int_to_codechr(pt_protein->aa[aa_ct].actual),
					pt_protein->aa[aa_ct].chain_id);
			}
			if (print_score == TRUE)
				print_score_table(aa_ct, pt_protein);
			fprintf(STDOUT, "Np_protein :     total score = %8.2f,  num_topscorer = %2d\n",
				pt_protein->aa[aa_ct].num_valid_scores,
				pt_protein->aa[aa_ct].num_topscorer);
			if (print_predict == TRUE) {
				print_prediction(aa_ct, pt_protein);
				fprintf(STDOUT, "Np_protein :     accuracy = %6.4f\n",
					pt_protein->aa[aa_ct].accuracy);
			}
		}
	}
	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "\n");
}
