/*
 *  np_print_pdb_file.c
 *  ProteinEncoder
 *
 *  Created by Naoto Morikawa on 2/23/07.
 *  Copyright 2007 Naoto Morikawa. All rights reserved.
 *
 *  2007-03-05 version 0.1 release
 *  2007-04-15 version 0.2 release
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "np_main.h"
#include "np_pdb_data.h"
#include "np_pe_protein.h"

extern struct pe_protein my_pe_protein;
extern struct pe_coor_system my_coor;

void print_pdb_line(struct pe_pdb_line * pt_line)
{
	fprintf(STDOUT, "name  |record_body\n");
	fprintf(STDOUT, "%.6s|%.74s\n", pt_line->record_name,
		pt_line->record_body);
	fflush(STDOUT);
}

void print_pdb_header(struct pe_pdb_header * pt_hd)
{
	fprintf(STDOUT,
		"name  |class                                   |date     |idcode\n");
	fprintf(STDOUT, "%.6s|%.40s|%.9s|%.4s\n", pt_hd->record_name,
		pt_hd->class, pt_hd->date, pt_hd->idcode);
	fflush(STDOUT);
}

void print_pdb_compnd(struct pe_pdb_compnd * pt_line)
{
/*    fprintf(STDOUT,"name  |c |spec_list\n"); */
	fprintf(STDOUT, "%.6s|%.2s|%.60s\n", pt_line->record_name,
		pt_line->continuation, pt_line->spec_list);
	fflush(STDOUT);
}

void print_pdb_model(struct pe_pdb_model * pt_line)
{
/*    fprintf(STDOUT,"name  |serial_num\n"); */
	fprintf(STDOUT, "%.6s|%.4s\n", pt_line->record_name,
		pt_line->serial_num);
	fflush(STDOUT);
}

void print_pdb_atom(struct pe_pdb_atom * pt_atm)
{
	fprintf(STDOUT,
		"name  |ser  |atm |l|res|c|seq |i|x-coor. |y-coor. |z-coor. |occu  |temp  |seg |el|chg\n");
	fprintf(STDOUT,
		"%.6s|%.5s|%.4s|%c|%.3s|%c|%.4s|%c|%.8s|%.8s|%.8s|%.6s|%.6s|%.4s|%.2s|%.2s\n",
		pt_atm->record_name, pt_atm->serial_num, pt_atm->atom_name,
		pt_atm->alt_loc, pt_atm->res_name, pt_atm->chain_id,
		pt_atm->res_seq, pt_atm->i_code, pt_atm->x, pt_atm->y,
		pt_atm->z, pt_atm->occupancy, pt_atm->temp_fac, pt_atm->seg_id,
		pt_atm->element, pt_atm->charge);
	fflush(STDOUT);
}

void print_pdb_ter(struct pe_pdb_ter * pt_line)
{
	fprintf(STDOUT, "name  |ser  |res|c|ser |i\n");
	fprintf(STDOUT, "%.6s|%.5s|%.3s|%c|%.4s|%c\n", pt_line->record_name,
		pt_line->serial_num,
		pt_line->res_name,
		pt_line->chain_id, pt_line->res_serial_num, pt_line->i_code);
	fflush(STDOUT);
}

void print_pdb_master(struct pe_pdb_master * pt_line)
{
	fprintf(STDOUT,
		"name  |Rem  |zero |Het  |Hlx  |Sht  |Turn |Site |Xfor |Coor |Ter  |Cnct |Sqrs\n");
	fprintf(STDOUT,
		"%.6s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s|%.5s\n",
		pt_line->record_name, pt_line->num_remark, pt_line->zeros,
		pt_line->num_het, pt_line->num_helix, pt_line->num_sheet,
		pt_line->num_turn, pt_line->num_site, pt_line->num_xform,
		pt_line->num_coor, pt_line->num_ter, pt_line->num_conect,
		pt_line->num_seqres);
	fflush(STDOUT);
}

void print_pe_protein(struct pe_protein * pt_strct)
{
	int i;

	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "my_pe_protein: mod |mol |chn |seq |res |atm |\n");
	fprintf(STDOUT, "my_pe_protein: %4d|%4d|%4d|%4d|%4d|%4d|\n", pt_strct->mod_num,
		pt_strct->mol_num,
		pt_strct->chn_num,
		pt_strct->seq_num, pt_strct->res_num, pt_strct->atm_num);
/*
	fprintf(STDOUT,"my_pe_protein: tiles/res|tile_max |dist_fac\n");
    fprintf(STDOUT,"my_pe_protein: %9.4f|%9d|%9.4f\n", pt_strct->tiles_per_res, pt_strct->tile_max_num, pt_strct->dist_factor);
*/
	if (pt_strct->model != NULL) {
		fprintf(STDOUT, "my_pe_protein: <models>\n");
		fprintf(STDOUT, "my_pe_protein:            [seq_head, seq_tail]\n");
		for (i = 1; i <= pt_strct->mod_num; i++) {
			fprintf(STDOUT, "my_pe_protein: model[%2d]= [ %7d,  %7d]\n", i,
				pt_strct->model[i].seq_head,
				pt_strct->model[i].seq_tail);
		}
	}

	fprintf(STDOUT, "my_pe_protein: <molecules>\n");
	fprintf(STDOUT, "my_pe_protein:                [chn_head, chn_tail] id name\n");
	for (i = 1; i <= pt_strct->mol_num; i++) {
		fprintf(STDOUT, "my_pe_protein: molecules[%2d]= [ %7d,  %7d] %2d %s\n",
			i, pt_strct->molecule[i].chain_head,
			pt_strct->molecule[i].chain_tail,
			pt_strct->molecule[i].id, pt_strct->molecule[i].name);
	}

	fprintf(STDOUT, "my_pe_protein: <chains>\n");
	fprintf(STDOUT,
		"my_pe_protein:            [atm_head, atm_tail] chn_id |mol_id |\n");
	for (i = 1; i <= pt_strct->chn_num; i++) {
		fprintf(STDOUT, "my_pe_protein: chain[%2d]= [ %7d,  %7d] %7c|%7d|\n", i,
			pt_strct->chain[i].atom_head,
			pt_strct->chain[i].atom_tail,
			pt_strct->chain[i].id, pt_strct->chain[i].mol_id);
	}

	if (pt_strct->sequence != NULL) {
		fprintf(STDOUT, "my_pe_protein: <sequences>\n");
		fprintf(STDOUT,
			"my_pe_protein:                [atm_head, atm_tail] chn_id |mol_id |mod_num [tile_head,tile_tail]\n");
		for (i = 1; i <= pt_strct->seq_num; i++) {
			fprintf(STDOUT,
				"my_pe_protein: sequence[%3d]= [ %7d,  %7d] %7c|%7d|%7d [ %8d, %8d]\n",
				i, pt_strct->sequence[i].atom_head,
				pt_strct->sequence[i].atom_tail,
				pt_strct->sequence[i].id,
				pt_strct->sequence[i].mol_id,
				pt_strct->sequence[i].mod_num,
				pt_strct->sequence[i].tile_head,
				pt_strct->sequence[i].tile_tail);
		}
	}

	fprintf(STDOUT, "my_pe_protein: end of my_pe_protein \n");
	fprintf(STDOUT, "\n");
	fflush(STDOUT);
}

void print_pe_atom(struct pe_atom * pt_strct)
{
	fprintf(STDOUT, "\n");
	fprintf(STDOUT,
		"atom : ser. name res chn num (       x,        y,        z)    occu. id\n");
	fprintf(STDOUT,
		"atom : %4d %4d %.3s  %c  %4d (%8.3f, %8.3f, %8.3f) %8f %.4s\n",
		pt_strct->serial_num, pt_strct->atom_name, pt_strct->res_name,
		pt_strct->chain_id, pt_strct->res_seq, pt_strct->x, pt_strct->y,
		pt_strct->z, pt_strct->occupancy, pt_strct->seg_id);
	fprintf(STDOUT, "\n");
	fflush(STDOUT);
}

void print_pe_sequences(void)
{
	int mod_ct, seq_ct, atm_ct;

	fprintf(STDOUT, "\n");
	fprintf(STDOUT, "seq. : #sequences= %d, #molecules= %d, #models= %d \n",
		my_pe_protein.seq_num, my_pe_protein.mol_num, my_pe_protein.mod_num);

	for (mod_ct = 1; mod_ct <= my_pe_protein.mod_num; mod_ct++) {
		for (seq_ct = my_pe_protein.model[mod_ct].seq_head;
		     seq_ct <= my_pe_protein.model[mod_ct].seq_tail; seq_ct++) {
			fprintf(STDOUT,
				"seq. : #seq= %d (from %d-th atom to %d-th atom)\n",
				seq_ct, my_pe_protein.sequence[seq_ct].atom_head,
				my_pe_protein.sequence[seq_ct].atom_tail);
			fprintf(STDOUT,
				"seq. : [ atom ]  ser.  name res chn num  (       x,        y,        z) occu. id\n");
			for (atm_ct = my_pe_protein.sequence[seq_ct].atom_head;
			     atm_ct <= my_pe_protein.sequence[seq_ct].atom_tail;
			     atm_ct++) {
				fprintf(STDOUT, "seq. : [%6d] %6d ", atm_ct,
					my_pe_protein.atom[atm_ct].serial_num);
				if (my_pe_protein.atom[atm_ct].atom_name == PE_CC)
					fprintf(STDOUT, " C  ");
				else if (my_pe_protein.atom[atm_ct].atom_name == PE_CA)
					fprintf(STDOUT, " CA ");
				else if (my_pe_protein.atom[atm_ct].atom_name == PE_NN)
					fprintf(STDOUT, " N  ");
				else
					fprintf(STDOUT, " UN ");

				fprintf(STDOUT,
					" %.3s  %c  %4d (%8.3f, %8.3f, %8.3f) %4.3f %.4s\n",
					my_pe_protein.atom[atm_ct].res_name,
					my_pe_protein.atom[atm_ct].chain_id,
					my_pe_protein.atom[atm_ct].res_seq,
					my_pe_protein.atom[atm_ct].x,
					my_pe_protein.atom[atm_ct].y,
					my_pe_protein.atom[atm_ct].z,
					my_pe_protein.atom[atm_ct].occupancy,
					my_pe_protein.atom[atm_ct].seg_id);
			}
			fprintf(STDOUT, "seq. : \n");
		}
	}
	fprintf(STDOUT, "seq. : end of sequence\n");
	fprintf(STDOUT, "\n");
	fflush(STDOUT);
}
