/*
 *  np_print_prediction.c
 *  NtileCodePredictor
 *
 *  Created by Naoto Morikawa on 2/24/07.
 *  Copyright 2007 (C) 2007 Naoto Morikawa. All rights reserved.
 *
 *  2007-03-05 version 0.1 release
 *  2007-03-12 fix a misspelling of "Genaral" in print_pred_remark()
 *  2007-04-10 support the design-protein option, "-d", in print_pred_pred(), 
 *              print_pred_stats(), print_pred_file()
 *              (output "***_dsgn.pred" in the case of "-d")
 *  2007-04-10 change the entry name of [Actu] to [Code] in print_pred_actu()
 *  2007-04-10 change [pred], [accu], ..., to [Pred], [Accu], ... in print_pred_remark()
 *  2007-04-10 fix a minor bug in print_pred_comp()
 *  2007-04-11 create print_pred_hit() to support the [Hit.] entry
 *  2007-04-11 support the [Hit.] entry print_pred_remark(), print_predicted_code(), print_predicted_res()
 *  2007-04-10 support the check-structure option, "-c", in print_pred_remark(), 
 *              print_pred_stats(), print_pred_file()
 *              (output "***_chck.pred" in the case of "-c")
 *  2007-04-15 version 0.2 release
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "np_main.h"
#include "np_print.h"

extern int int_to_aachr(int val);	// np_search_table.c
extern int int_to_codechr(int val);	// np_print.c
extern struct np_settings my_settings;

#define MAX_SEQ_WIDTH	100

void print_pred_res(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Res.] ");
	else
		fprintf(fp_code, "[Res.] + ");
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		fprintf(fp_code, "%c", int_to_aachr(pt_protein->aa[aa_ct].name));
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

void print_pred_pred(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Pred] ");
	else
		fprintf(fp_code, "[Pred] + ");
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		if (pt_protein->aa[aa_ct].num_topscorer > 1) {
			fprintf(fp_code, "*");
			pt_protein->mult_ct++;
		} else if (pt_protein->aa[aa_ct].num_topscorer <= 0) {
			fprintf(fp_code, "-");
		} else {
			if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
				fprintf(fp_code, "%c", int_to_codechr(PRED_CODE(aa_ct, 0)));
			} else {
				fprintf(fp_code, "%c", int_to_aachr(PRED_CODE(aa_ct, 0)));
			}
		}
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

void print_pred_mult(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Mult] ");
	else
		fprintf(fp_code, "[Mult] + ");
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		if (pt_protein->aa[aa_ct].num_topscorer > 9)
			fprintf(fp_code, "U");
		else if (pt_protein->aa[aa_ct].num_topscorer > 1)
			fprintf(fp_code, "%1d", pt_protein->aa[aa_ct].num_topscorer);
		else if (pt_protein->aa[aa_ct].num_topscorer <= 0)
			fprintf(fp_code, "-");
		else
			fprintf(fp_code, "_");
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

void print_pred_accu(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Accu] ");
	else
		fprintf(fp_code, "[Accu] + ");
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		if (pt_protein->aa[aa_ct].accuracy >= 0.9)
			fprintf(fp_code, "9");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.8)
			fprintf(fp_code, "8");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.7)
			fprintf(fp_code, "7");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.6)
			fprintf(fp_code, "6");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.5)
			fprintf(fp_code, "5");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.4)
			fprintf(fp_code, "4");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.3)
			fprintf(fp_code, "3");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.2)
			fprintf(fp_code, "2");
		else if (pt_protein->aa[aa_ct].accuracy >= 0.1)
			fprintf(fp_code, "1");
		else if (pt_protein->aa[aa_ct].accuracy > 0.0)
			fprintf(fp_code, "0");
		else
			fprintf(fp_code, "-");
		pt_protein->ave_accuracy += pt_protein->aa[aa_ct].accuracy;
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

// NM 2007-04-11
void print_pred_hit(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Hit.] ");
	else
		fprintf(fp_code, "[Hit.] + ");
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		if (pt_protein->aa[aa_ct].num_valid_scores >= 100)
			fprintf(fp_code, "U");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 90)
			fprintf(fp_code, "9");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 80)
			fprintf(fp_code, "8");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 70)
			fprintf(fp_code, "7");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 60)
			fprintf(fp_code, "6");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 50)
			fprintf(fp_code, "5");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 40)
			fprintf(fp_code, "4");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 30)
			fprintf(fp_code, "3");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 20)
			fprintf(fp_code, "2");
		else if (pt_protein->aa[aa_ct].num_valid_scores >= 10)
			fprintf(fp_code, "1");
		else if (pt_protein->aa[aa_ct].num_valid_scores > 0)
			fprintf(fp_code, "0");
		else
			fprintf(fp_code, "-");
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

void print_pred_comp(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Comp] ");
	else
		fprintf(fp_code, "[Comp] + ");
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		if (pt_protein->aa[aa_ct].num_topscorer > 1) {
			int tbl_ct;
			float max_score;	// bug fix NM 2007-04-10
			BOOLEAN hit_flag;
			
			hit_flag = FALSE;
			max_score = PRED_SCORE(aa_ct, 0);
			for (tbl_ct = 1; tbl_ct < ST_UNK - 1; tbl_ct++) {
				if (FLOAT_NEQ(PRED_SCORE(aa_ct, tbl_ct) - max_score))
					continue;
				hit_flag = TRUE;
				break;
			}
			if (hit_flag == TRUE) {
				pt_protein->aa[aa_ct].comp_result = R_CONTAINED;
				fprintf(fp_code, "c");
				pt_protein->contained_ct++;
			} else {
				pt_protein->aa[aa_ct].comp_result = R_FAIL;
				fprintf(fp_code, "F");
			}
		} else if (pt_protein->aa[aa_ct].num_topscorer <= 0) {
			// pt_protein->aa[aa_ct].comp_result =R_NOPREDICTION;
			fprintf(fp_code, "-");
		} else if ((my_settings.design_protein == FALSE)  && 
			   (pt_protein->aa[aa_ct].actual == PRED_CODE(aa_ct, 0))) {
			pt_protein->aa[aa_ct].comp_result = R_SUCCESS;
			fprintf(fp_code, "_");
			pt_protein->success_ct++;
		} else if ((my_settings.design_protein == TRUE)  && 
			   (pt_protein->aa[aa_ct].name == PRED_CODE(aa_ct, 0))) {
			pt_protein->aa[aa_ct].comp_result = R_SUCCESS;
			fprintf(fp_code, "_");
			pt_protein->success_ct++;
		} else {
			pt_protein->aa[aa_ct].comp_result = R_FAIL;
			fprintf(fp_code, "F");
		}
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

void print_pred_actu(int aa_init, struct np_protein * pt_protein, FILE * fp_code)
{
	int chn_ind;
	int aa_tail;
	int aa_ct;
	
	chn_ind = pt_protein->aa[aa_init].chain_id;
	if ( pt_protein->chain[chn_ind].aa_tail <= aa_init + MAX_SEQ_WIDTH)
		aa_tail = pt_protein->chain[chn_ind].aa_tail + 1;
	else 
		aa_tail = aa_init + MAX_SEQ_WIDTH;
	
	if ( pt_protein->chain[chn_ind].aa_head == aa_init)
		fprintf(fp_code, "[Code] ");	// NM 2007-04-10
	else
		fprintf(fp_code, "[Code] + ");	// NM 2007-04-10
	for (aa_ct = aa_init;
	     aa_ct < aa_tail;
	     aa_ct++) {
		fprintf(fp_code, "%c", int_to_codechr(pt_protein->aa[aa_ct].actual));
	}
	fprintf(fp_code, " : chn%2d\n", chn_ind);
}

void print_pred_stats(struct np_protein * pt_protein, FILE * fp_code)
{
	float all_target;
	int cutoff_num;
	int chn_ct, aa_ct;
	int num_valid_codes;
	
	if (my_settings.check_structure == TRUE) {	// NM 2007-04-11
		cutoff_num = 2 * ((int) my_settings.tiles_per_res - 1) / 2;
	} else if (my_settings.design_protein == TRUE) {
		cutoff_num = 0;
	} else {			// NM 2007-04-10
		cutoff_num = 2 * 2;	// Discard two amino acids at both ends of the chain
	}
	all_target = (float) (pt_protein->aa_num - pt_protein->chain_num * cutoff_num);

	pt_protein->success_rate = (float) pt_protein->success_ct / all_target;
	pt_protein->extended_success_rate = (float)
		(pt_protein->success_ct + pt_protein->contained_ct) / all_target;
	pt_protein->mult_candidate_rate = (float) pt_protein->mult_ct / all_target;
	pt_protein->coverage = (float) pt_protein->hit_ct / all_target;
	if (0) {
		fprintf(STDOUT, "print_pred_stats() : DEBUG pt_protein->aa_num = %d\n",
			pt_protein->aa_num);
		fprintf(STDOUT, "print_pred_stats() : DEBUG pt_protein->success_ct = %d\n",
			pt_protein->success_ct);
		fprintf(STDOUT, "print_pred_stats() : DEBUG pt_protein->mult_ct = %d\n",
			pt_protein->mult_ct);
		fprintf(STDOUT, "print_pred_stats() : DEBUG pt_protein->contained_ct = %d\n",
			pt_protein->contained_ct);
		fprintf(STDOUT, "print_pred_stats() : DEBUG pt_protein->hit_ct = %d\n",
			pt_protein->hit_ct);
	}
	
	num_valid_codes = 0;
	for (chn_ct = pt_protein->chain_head;
	     chn_ct <= pt_protein->chain_tail; chn_ct++) {
		for (aa_ct = pt_protein->chain[chn_ct].aa_head;
		     aa_ct <= pt_protein->chain[chn_ct].aa_tail;
		     aa_ct++) {
			if (pt_protein->aa[aa_ct].num_topscorer > 0)
				num_valid_codes++;
		}
	}
	if (num_valid_codes != 0)
		pt_protein->ave_accuracy = pt_protein->ave_accuracy / (float) num_valid_codes;
	else
		pt_protein->ave_accuracy = 0.0;
	
	fprintf(fp_code, "Statistics\n");
	// NM 2007-04-11 change %1d ==> %2d to support large numbers
	if (my_settings.compare_enabled == TRUE) {
		fprintf(fp_code, "  Success rate          = %6.4f    (= #'_' / (#aa - #chn * %2d))          : %s\n",
			pt_protein->success_rate, cutoff_num, pt_protein->filename);
		fprintf(fp_code, "  Extended success rate = %6.4f    (= (#'_' + #'c') / (#aa - #chn * %2d)) : %s\n",
			pt_protein->extended_success_rate, cutoff_num, pt_protein->filename);
	}
	fprintf(fp_code, "  Coverage              = %6.4f    (= (#aa - #'-') / (#aa - #chn * %2d))  : %s\n",
		pt_protein->coverage, cutoff_num, pt_protein->filename);
	fprintf(fp_code, "  Mult candidate rate   = %6.4f    (= #'*' / (#aa - #chn * %2d))          : %s\n",
		pt_protein->mult_candidate_rate, cutoff_num, pt_protein->filename);
	fprintf(fp_code, "  Ave. accuracy         = %6.4f                                          : %s\n",
		pt_protein->ave_accuracy, pt_protein->filename);
}

void print_pred_remark(FILE * fp_code)
{
	int cutoff_length;
	
	if (my_settings.check_structure == TRUE) {	// NM 2007-04-11
		cutoff_length = ((int) my_settings.tiles_per_res - 1) / 2;
	} else if (my_settings.design_protein == TRUE) {
		cutoff_length = 0;
	} else {			// NM 2007-04-10
		cutoff_length = 2;	// Discard two amino acids at both ends of the chain
	}
	
	fprintf(fp_code, "// Remarks\n");
	fprintf(fp_code, "// (0) General \n");		// 2007-03-12
	fprintf(fp_code, "//     'The [Res.] and the [Code] entries show the actual sequences of residues and codes respectively.\n");	// NM 2007-04-11
	fprintf(fp_code, "//     '-' means no entry in the frag_code.tbl table.\n");
	fprintf(fp_code, "// (1) Statistics \n");
		// NM 2007-04-11 change %1d ==> %2d to support large numbers
	fprintf(fp_code, "//     %1d amino acids at both ends of chains are excluded from computation.\n", cutoff_length);
	fprintf(fp_code, "//     '#aa' denotes the number of all amino acids.\n");
	fprintf(fp_code, "//     '#chn' denotes the number of all chains.\n");
	if (my_settings.design_protein == TRUE) {	// NM 2007-04-11
		fprintf(fp_code, "// (2) [Pred] entry \n");	// NM 2007-04-10
		fprintf(fp_code, "//     The predicted amino-acid sequence is shwon.\n");
		fprintf(fp_code, "//     '*' means multiple candidates for the position.\n");
		fprintf(fp_code, "// (3) [Hit.] entry \n");
		fprintf(fp_code, "//     Hit count = #{occurrence of the code-fragment in the conversion table}\n");
		fprintf(fp_code, "//     'n' means hit count is greater than or equal to n * 10.\n");
		fprintf(fp_code, "//     'U' means hit count is greater than 99.\n");
		fprintf(fp_code, "// (4) [Accu] entry \n");
		fprintf(fp_code, "//     Accuracy = #{occurrence of the predicted amino-acid-fragment in the conversion table} / \n");	// NM 2007-04-10
		fprintf(fp_code, "//                           #{occurrence of the code-fragment in the table}. \n");
		fprintf(fp_code, "//     (In the case of multiple candidates, the occurence of one of them is considered.)\n");
		fprintf(fp_code, "//     And 'n' means prediction accuracy is greater than or equal to n / 10.\n");
	} else {
		fprintf(fp_code, "// (2) [Pred] entry \n");	// NM 2007-04-10
		fprintf(fp_code, "//     The predicted code sequence is shwon.\n");
		fprintf(fp_code, "//     '*' means multiple candidates for the position.\n");
		fprintf(fp_code, "// (3) [Hit.] entry \n");
		fprintf(fp_code, "//     Hit count = #{occurrence of the amino-acid-fragment in the conversion table}\n");
		fprintf(fp_code, "//     'n' means hit count is greater than or equal to n * 10.\n");
		fprintf(fp_code, "//     'U' means hit count is greater than 99.\n");
		fprintf(fp_code, "// (4) [Accu] entry \n");
		fprintf(fp_code, "//     Accuracy = #{occurrence of the predicted code-fragment in the conversion table} / \n");	// NM 2007-04-10
		fprintf(fp_code, "//                           #{occurrence of the amino-acid-fragment in the table}. \n");
		fprintf(fp_code, "//     (In the case of multiple candidates, the occurence of one of them is considered.)\n");
		fprintf(fp_code, "//     And 'n' means prediction accuracy is greater than or equal to n / 10.\n");
	}
	fprintf(fp_code, "// (5) [Mult] entry \n");	// NM 2007-04-10
	fprintf(fp_code, "//     '_' means single candidate.\n");
	fprintf(fp_code, "//     'n' denotes the number of candidates.\n");
	fprintf(fp_code, "//     'U' means the number of candidates is greater than 9.\n");
	if (my_settings.compare_enabled == TRUE) {
		fprintf(fp_code, "// (6) [Comp] entry \n");	// NM 2007-04-10
		fprintf(fp_code, "//     '_' means successful prediction.\n");
		fprintf(fp_code, "//     'F' means prediction failure.\n");
		fprintf(fp_code, "//     'c' means the actual code is contained in the condidates.\n");
	}
}

// NM 2007-04-10
void print_predicted_res(int chn_ct, struct np_protein * pt_protein, 
			 FILE * fp_code)
{
	int aa_ct;
	
	for (aa_ct = pt_protein->chain[chn_ct].aa_head;
	     aa_ct <= pt_protein->chain[chn_ct].aa_tail;
	     aa_ct += MAX_SEQ_WIDTH) {
		print_pred_actu(aa_ct, pt_protein, fp_code);
		print_pred_pred(aa_ct, pt_protein, fp_code);
		print_pred_hit(aa_ct, pt_protein, fp_code);	// NM 2007-04-11
		print_pred_accu(aa_ct, pt_protein, fp_code);
		print_pred_mult(aa_ct, pt_protein, fp_code);
		if (my_settings.compare_enabled == TRUE) {
			print_pred_comp(aa_ct, pt_protein, fp_code);
			print_pred_res(aa_ct, pt_protein, fp_code);
		}
		fprintf(fp_code, "\n");
	}
	return;
}

void print_predicted_code(int chn_ct, struct np_protein * pt_protein,
			  FILE * fp_code)
{
	int aa_ct;
	
	for (aa_ct = pt_protein->chain[chn_ct].aa_head;
	     aa_ct <= pt_protein->chain[chn_ct].aa_tail;
	     aa_ct += MAX_SEQ_WIDTH) {
		print_pred_res(aa_ct, pt_protein, fp_code);
		print_pred_pred(aa_ct, pt_protein, fp_code);
		print_pred_hit(aa_ct, pt_protein, fp_code);	// NM 2007-04-11
		print_pred_accu(aa_ct, pt_protein, fp_code);
		print_pred_mult(aa_ct, pt_protein, fp_code);
		if (my_settings.compare_enabled == TRUE) {
			print_pred_comp(aa_ct, pt_protein, fp_code);
			print_pred_actu(aa_ct, pt_protein, fp_code);
		}
		fprintf(fp_code, "\n");
	}
	return;
}

// #define EXT_LENGTH (1+1)	/* ***.ent => ***.pred + '\0' */	// NM 2007-04-10
#define EXT_LENGTH (6+1)	// NM 2007-04-10: ***.ent => ***_dsgn.pred + '\0'

void print_pred_file(const char * filename, struct np_protein * pt_protein)
{
	FILE * fp_code;
	char * outfile;		// .pred file
	char * s;		// ptr to file extension
	int str_size;
	int chn_ct;
	
	// open .pred file
	str_size = (int) strlen(filename);
	outfile = (char *) malloc((str_size + EXT_LENGTH) * sizeof(char));
	if (outfile == NULL) {
		fprintf(STDERR, 
			"print_pred_file() : ERROR outfile allocation failed\n");
		return;
	}
	strcpy(outfile, filename);
	s = strrchr(outfile, '.');	/* s = ".ent" */
	if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
		strcpy(s+1, "pred\0");
	} else if (my_settings.check_structure == TRUE) {	// NM 2007-04-11
		strcpy(s, "_chck.pred\0");
	} else {
		strcpy(s, "_dsgn.pred\0");
	}
	fprintf(STDOUT, "PREDICTION FILE : %s\n", outfile);
	if ((fp_code = fopen(outfile, "w")) == NULL) {
		fprintf(STDERR, "print_pred_file() : can't open %s\n", s);
		if (outfile != NULL) free(outfile);
		return;
	}
	
	// main loop
	for (chn_ct = pt_protein->chain_head; chn_ct <= pt_protein->chain_tail;
	     chn_ct++) {
		if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
			print_predicted_code(chn_ct, pt_protein, fp_code);
		} else {
			print_predicted_res(chn_ct, pt_protein, fp_code);
		}
	}
	print_pred_stats(pt_protein, fp_code);
	fprintf(fp_code, "\n");
	if (DEBUG) print_pred_remark(fp_code);

	if (fclose(fp_code)) {
		fprintf(STDERR, "print_pred_file() : Can't close %s\n", outfile);
		abort();
	}
	if (outfile != NULL) free(outfile);
	return;
}

