/*
 *  np_search_table.c
 *  NtileCodePredictor
 *
 *  Created by Naoto Morikawa on 2/22/07.
 *  Copyright (C) 2007 Naoto Morikawa. All rights reserved.
 *
 *  2007-03-05 version 0.1 release
 *  2007-04-10 support the design-protein option, "-d", in search_frag_code_table()
 *  2007-04-10 create search_table_for_code_fragment() 
 *              to support the "-d" option
 *  2007-04-10 add the case of L_ELSE in int_to_aachr()
 *  2007-04-11 support the check-structure option, "-c", in search_table_for_code_fragment()
 *  2007-04-15 version 0.2 release
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "np_main.h"
#include "np_print.h"

extern int int_to_codechr(int val);		// np_print.c
extern int aachr_to_int(char * aa_name);	// NM 2007-04-10
extern struct np_settings my_settings;		// NM 2007-04-10

int codechr_to_int(char * code_chr)
{
	switch (*code_chr) {
		case ('0'):
			return ST_0;
		case ('1'):
			return ST_1;
		case ('2'):
			return ST_2;
		case ('3'):
			return ST_3;
		case ('4'):
			return ST_4;
		case ('5'):
			return ST_5;
		case ('6'):
			return ST_6;
		case ('7'):
			return ST_7;
		case ('8'):
			return ST_8;
		case ('9'):
			return ST_9;
		case ('A'):
			return ST_A;
		case ('B'):
			return ST_B;
		case ('C'):
			return ST_C;
		case ('D'):
			return ST_D;
		case ('E'):
			return ST_E;
		case ('F'):
			return ST_F;
		case ('G'):
			return ST_G;
		case ('H'):
			return ST_H;
		case ('I'):
			return ST_I;
		case ('J'):
			return ST_J;
		case ('K'):
			return ST_K;
		case ('L'):
			return ST_L;
		case ('M'):
			return ST_M;
		case ('N'):
			return ST_N;
		case ('O'):
			return ST_O;
		case ('P'):
			return ST_P;
		case ('Q'):
			return ST_Q;
		case ('R'):
			return ST_R;
		case ('S'):
			return ST_S;
		case ('T'):
			return ST_T;
		case ('U'):
			return ST_U;
		case ('V'):
			return ST_V;
		case ('?'):
			return ST_UNK;
		case ('-'):
			return ST_UNDEF;
		default:
			fprintf(STDERR, "codechr_to_int() : ERROR invalid ntile code = '%c'\n", *code_chr);
			return ST_INV;
	}
}

int int_to_aachr(int val)
{
	switch (val) {
		case (L_GLY):
			return 'G';
		case (L_ALA):
			return 'A';
		case (L_VAL):
			return 'V';
		case (L_PHE):
			return 'F';
		case (L_PRO):
			return 'P';
		case (L_HYP):
			return '-';
		case (L_MET):
			return 'M';
		case (L_ILE):
			return 'I';
		case (L_LEU):
			return 'L';
		case (L_ASP):
			return 'D';
		case (L_GLU):
			return 'E';
		case (L_LYS):
			return 'K';
		case (L_ARG):
			return 'R';
		case (L_SER):
			return 'S';
		case (L_THR):
			return 'T';
		case (L_TYR):
			return 'Y';
		case (L_HIS):
			return 'H';
		case (L_CYS):
			return 'C';
		case (L_ASN):
			return 'N';
		case (L_GLN):
			return 'Q';
		case (L_TRP):
			return 'W';
		case (L_ASX):
			return 'B';
		case (L_GLX):
			return 'Z';
		case (L_UNK):
			return '=';
		case (L_ELSE):		// NM 2007-04-10
			return '?';
		default:
			fprintf(STDERR, "int_to_aachr() : ERROR invalid amino acid code value = %d\n", val);
			return '?';
	}
}

#define MAX_BUFF_SIZE 256	// 123456789:123456789'\n''\0'  (frag:code)

int count_fragment_length(FILE * fp_tbl)
{
	char tbl_line[MAX_BUFF_SIZE];
	int tbl_ind;
	int length;
	int c;
	
	while (fgets(tbl_line, MAX_BUFF_SIZE, fp_tbl) != NULL) {
		tbl_ind = 0;
		while (tbl_line[tbl_ind] == ' ') tbl_ind++;
		if (tbl_line[tbl_ind] == '/')	// commnent line
			continue;
		length = 0;
		while ( (c = getc(fp_tbl)) != EOF) {
			if (c == ':') {
				return length;
			} else {
				length++;
			}
		}
		break;
	}
	fprintf(STDERR, "count_fragment_length() : Format error (missing ':')\n");
	return -1;
}

// NM 2007-04-10
void search_table_for_code_fragment(FILE * fp_tbl, struct np_fragment * pt_fragment,
			       struct np_protein * pt_protein)
{	
	char tbl_line[MAX_BUFF_SIZE];
	int chain_length;
	int aa_ind, tbl_ind;
	int ind_of_code_top;
	int code_val;
	int hit_ct;
	int miss_ct;
	BOOLEAN hit_flag;
	
	// Fragment length check
	chain_length = pt_protein->chain[pt_fragment->chn_ct].aa_tail
		- pt_fragment->aa_ct + 1;
	if (chain_length < pt_fragment->length) return;
	
	if (0) print_code_fragment(pt_fragment, pt_protein);
	fseek(fp_tbl, 0L, SEEK_SET);
	hit_ct = 0;
	miss_ct = 0;
	ind_of_code_top = (int) my_settings.tiles_per_res + 1;	// "ABCDE:01234"
	while (fgets(tbl_line, MAX_BUFF_SIZE, fp_tbl) != NULL) {
		if (0)
			fprintf(STDOUT, "search_table_for_code_fragment() : current line = %s",
				tbl_line);
		hit_flag = TRUE;
		tbl_ind = ind_of_code_top;	// Initial poisition
		for (aa_ind = pt_fragment->aa_ct;
		     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
		     aa_ind++, tbl_ind++) {
			if (0) {
				fprintf(STDOUT, "search_table_for_code_fragment() : tbl[%d] = %c\n",
					tbl_ind, tbl_line[tbl_ind]);
			}
			if (pt_protein->aa[aa_ind].actual >= ST_UNK) {
				continue;	// skip if the code is '?' or '-'
			}
			if (tbl_line[tbl_ind] != int_to_codechr(pt_protein->aa[aa_ind].actual)) {
				hit_flag = FALSE;
				break;
			}
		}
		if (hit_flag == TRUE) {		// Hit case (renew score tables)
			if (0)
				fprintf(STDOUT, "search_table_for_code_fragment() : hit line = %s",
					tbl_line);
			tbl_ind = 0;		// position of the top amino acid
			if (my_settings.check_structure == TRUE) {
				int mid_point;
				
				mid_point = ((int) my_settings.tiles_per_res - 1) / 2;
				aa_ind = pt_fragment->aa_ct + mid_point;
				code_val = aachr_to_int(&tbl_line[tbl_ind]);
				pt_protein->aa[aa_ind].score_tbl[code_val] += 1.0;
				if ( code_val < L_UNK) {
					pt_protein->aa[aa_ind].num_valid_scores += 1.0;
				}
			} else {
				for (aa_ind = pt_fragment->aa_ct;
				     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
				     aa_ind++, tbl_ind++) {
					code_val = aachr_to_int(&tbl_line[tbl_ind]);
					pt_protein->aa[aa_ind].score_tbl[code_val] += 1.0;
					if ( code_val < L_UNK) {
						pt_protein->aa[aa_ind].num_valid_scores += 1.0;
					}
				}
			}
			hit_ct++;
		} else {
			if(0) miss_ct++;
		}
	}
	if (DEBUG) {
		if (hit_ct == 0) {
			int chn_ind;
			int aa_pos_offset;
			
			fprintf(STDOUT, "search_table_for_code_fragment() : No hit for code fragment = \"");
			for (aa_ind = pt_fragment->aa_ct;
			     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
			     aa_ind++) {
				fprintf(STDOUT, "%c",
					int_to_codechr(pt_protein->aa[aa_ind].actual));
			}
			chn_ind = pt_protein->aa[pt_fragment->aa_ct].chain_id;
			aa_pos_offset = pt_protein->chain[chn_ind].aa_head - 1;
			fprintf(STDOUT, "\"  at  aa%4d of chain%2d\n",
				pt_fragment->aa_ct - aa_pos_offset, chn_ind);
		} else {
			if (0) {
				fprintf(STDOUT, "search_table_for_code_fragment() : hit_ct = %d  (miss_ct = %d)\n",
					hit_ct, miss_ct);
			}
		}
	}
	return;
}

void search_table_for_fragment(FILE * fp_tbl, struct np_fragment * pt_fragment,
			       struct np_protein * pt_protein)
{	
	char tbl_line[MAX_BUFF_SIZE];
	int chain_length;
	int aa_ind, tbl_ind;
	int code_val;
	int hit_ct;
	int miss_ct;
	BOOLEAN hit_flag;
	
	// Fragment length check
	chain_length = pt_protein->chain[pt_fragment->chn_ct].aa_tail
		- pt_fragment->aa_ct + 1;
	if (chain_length < pt_fragment->length) return;
	
	if (0) print_fragment(pt_fragment, pt_protein);
	fseek(fp_tbl, 0L, SEEK_SET);
	hit_ct = 0;
	miss_ct = 0;
	while (fgets(tbl_line, MAX_BUFF_SIZE, fp_tbl) != NULL) {
		if (0)
			fprintf(STDOUT, "search_table_for_fragment() : current line = %s",
				tbl_line);
		hit_flag = TRUE;
		tbl_ind = 0;
		for (aa_ind = pt_fragment->aa_ct;
		     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
		     aa_ind++, tbl_ind++) {
			if (tbl_line[tbl_ind] != int_to_aachr(pt_protein->aa[aa_ind].name)) {
				hit_flag = FALSE;
				break;
			}
		}
		if (hit_flag == TRUE) {		// Hit case (renew score tables)
			if (0)
				fprintf(STDOUT, "search_table_for_fragment() : hit line = %s",
					tbl_line);
			tbl_ind++;		// skip ':'
			for (aa_ind = pt_fragment->aa_ct;
			     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
			     aa_ind++, tbl_ind++) {
				code_val = codechr_to_int(tbl_line + tbl_ind);
				pt_protein->aa[aa_ind].score_tbl[code_val] += 1.0;
				if ( code_val < ST_UNK) {
					pt_protein->aa[aa_ind].num_valid_scores += 1.0;
				}
			}
			hit_ct++;
		} else {
			if(0) miss_ct++;
		}
	}
	if (DEBUG) {
		if (hit_ct == 0) {
			int chn_ind;
			int aa_pos_offset;
			
			fprintf(STDOUT, "search_table_for_fragment() : No hit for fragment = \"");
			for (aa_ind = pt_fragment->aa_ct;
			     aa_ind < pt_fragment->aa_ct + pt_fragment->length;
			     aa_ind++) {
				fprintf(STDOUT, "%c",
					int_to_aachr(pt_protein->aa[aa_ind].name));
			}
			chn_ind = pt_protein->aa[pt_fragment->aa_ct].chain_id;
			aa_pos_offset = pt_protein->chain[chn_ind].aa_head - 1;
			fprintf(STDOUT, "\"  at  aa%4d of chain%2d\n",
				pt_fragment->aa_ct - aa_pos_offset, chn_ind);
		} else {
			if (0) {
				fprintf(STDOUT, "search_table_for_fragment() : hit_ct = %d  miss_ct = %d\n",
					hit_ct, miss_ct);
			}
		}
	}
	return;
}

/**
 * search_frag_code_table -  searchs the "frag_code.tbl" for all fragments of length N, 
 *		 If hit, renew the corresponding score tables.
 *
 */

void search_frag_code_table(char * table_name, struct np_protein * pt_protein)
{
	FILE * fp_frag_tbl;
	struct np_fragment cur_fragment;
	int chn_ct, aa_ct;
	
	// Load table
	fp_frag_tbl = fopen(table_name, "r");
	if (fp_frag_tbl == NULL) {
		fprintf(STDERR, "search_frag_code_table() : Can't open %s\n",
			table_name);
		return;
	}
	cur_fragment.length = count_fragment_length(fp_frag_tbl);
	if (DEBUG) {
		fprintf(STDERR, "\n");
		fprintf(STDERR, "search_frag_code_table() : fragment length = %d\n",
			cur_fragment.length);
	}
	
	// Search table
	for (chn_ct = pt_protein->chain_head;
	     chn_ct <= pt_protein->chain_tail; chn_ct++) {
		cur_fragment.chn_ct = chn_ct;
		for (aa_ct = pt_protein->chain[chn_ct].aa_head;
		     aa_ct <= pt_protein->chain[chn_ct].aa_tail
		     - cur_fragment.length + 1;
		     aa_ct++) {
			cur_fragment.aa_ct = aa_ct;
			if (my_settings.design_protein == FALSE) {	// NM 2007-04-10
				search_table_for_fragment(fp_frag_tbl, 
							  &cur_fragment,
							  pt_protein);
			} else {
				search_table_for_code_fragment(fp_frag_tbl, 
							       &cur_fragment, 
							       pt_protein);
			}
		}
	}
	
	if (fclose(fp_frag_tbl)) {
		fprintf(STDERR, "search_frag_code_table() : Can't close %s\n", 
			table_name);
		abort();
	}
	return;
}

